package cz.muni.fi.pa165.airportmanager.flightmodule.service;

import cz.muni.fi.pa165.airportmanager.domainmodule.dto.finddto.AirlineFindDto;
import cz.muni.fi.pa165.airportmanager.domainmodule.dto.finddto.PlaneFindDto;
import cz.muni.fi.pa165.airportmanager.domainmodule.exception.InvalidUserInputException;
import cz.muni.fi.pa165.airportmanager.domainmodule.exception.ObjectNotFoundException;
import cz.muni.fi.pa165.airportmanager.domainmodule.exception.ServerDownException;
import cz.muni.fi.pa165.airportmanager.domainmodule.service.RetrieveEntityService;
import cz.muni.fi.pa165.airportmanager.domainmodule.service.RetrieveEntityServiceImpl;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatusCode;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.WebClientRequestException;
import org.springframework.web.reactive.function.client.WebClientResponseException;

import java.util.UUID;

@Service
public class PlaneServiceImpl extends RetrieveEntityServiceImpl<PlaneFindDto>
        implements RetrieveEntityService<PlaneFindDto> {
    public PlaneServiceImpl(@Value("${plane.url}") String destinationUrl,
                            @Value("${plane.endpoint}") String destinationEndpoint) {
        super(destinationUrl, destinationEndpoint);
    }

    @Override
    public PlaneFindDto retrieve(UUID id, String authenticationHeader) {
        try {
            return retrieveResponseSpec(id, authenticationHeader)
                    .bodyToMono(PlaneFindDto.class)
                    .block();
        } catch (WebClientResponseException e) {
            if (e.getStatusCode() == HttpStatusCode.valueOf(404)) {
                throw new ObjectNotFoundException("Plane ID not found.");
            }

            if (e.getStatusCode() == HttpStatusCode.valueOf(400)) {
                throw new InvalidUserInputException("Invalid plane ID was provided.");
            }

            throw new ServerDownException("Unexpected error while communicating with the plane server.");
        } catch (WebClientRequestException e) {
            throw new ServerDownException("Plane server is down.");
        }
    }
}
